@@ -5,9 +5,8 @@
  */
 
 #include <libxml++/nodes/entityreference.h>
-#include <libxml++/exceptions/internal_error.h>
 
-#include <libxml/entities.h>
+#include <libxml/tree.h>
 
 namespace xmlpp
 {
