@@ -43,37 +43,37 @@ SaxParser::SaxParser(bool use_get_entity)
 {
   xmlSAXHandler temp = {
     SaxParserCallback::internal_subset,
-    0,  // isStandalone
-    0,  // hasInternalSubset
-    0,  // hasExternalSubset
-    0,  // resolveEntity
-    use_get_entity ? SaxParserCallback::get_entity : 0, // getEntity
+    nullptr, // isStandalone
+    nullptr, // hasInternalSubset
+    nullptr, // hasExternalSubset
+    nullptr, // resolveEntity
+    use_get_entity ? SaxParserCallback::get_entity : nullptr, // getEntity
     SaxParserCallback::entity_decl, // entityDecl
-    0,  // notationDecl
-    0,  // attributeDecl
-    0,  // elementDecl
-    0,  // unparsedEntityDecl
-    0,  // setDocumentLocator
+    nullptr, // notationDecl
+    nullptr, // attributeDecl
+    nullptr, // elementDecl
+    nullptr, // unparsedEntityDecl
+    nullptr, // setDocumentLocator
     SaxParserCallback::start_document, // startDocument
     SaxParserCallback::end_document, // endDocument
     SaxParserCallback::start_element, // startElement
     SaxParserCallback::end_element, // endElement
-    0,  // reference
+    nullptr, // reference
     SaxParserCallback::characters, // characters
-    0,  // ignorableWhitespace
-    0,  // processingInstruction
+    nullptr, // ignorableWhitespace
+    nullptr, // processingInstruction
     SaxParserCallback::comment,  // comment
     SaxParserCallback::warning,  // warning
     SaxParserCallback::error,  // error
     SaxParserCallback::fatal_error, // fatalError
-    0,  // getParameterEntity
+    nullptr, // getParameterEntity
     SaxParserCallback::cdata_block, // cdataBlock
-    0,  // externalSubset
-    0,  // initialized
-    0,  // private
-    0,  // startElementNs
-    0,  // endElementNs
-    0,  // serror
+    nullptr, // externalSubset
+    0,       // initialized
+    nullptr, // private
+    nullptr, // startElementNs
+    nullptr, // endElementNs
+    nullptr, // serror
   };
   *sax_handler_ = temp;
 
@@ -221,10 +221,10 @@ void SaxParser::parse_stream(std::istream& in)
 
   context_ = xmlCreatePushParserCtxt(
       sax_handler_.get(),
-      0, // user_data
-      0, // chunk
-      0, // size
-      0); // no filename for fetching external entities
+      nullptr,  // user_data
+      nullptr,  // chunk
+      0,        // size
+      nullptr); // no filename for fetching external entities
 
   if(!context_)
   {
@@ -233,10 +233,14 @@ void SaxParser::parse_stream(std::istream& in)
 
   initialize_context();
 
-  //TODO: Shouldn't we use a Glib::ustring here, and some alternative to std::getline()?
+  // std::string or Glib::ustring?
+  // Output from the XML parser is UTF-8 encoded.
+  // But the istream "in" is input, i.e. an XML file. It can use any encoding.
+  // If it's not UTF-8, the file itself must contain information about which
+  // encoding it uses. See the XML specification. Thus use std::string.
   int firstParseError = XML_ERR_OK;
   std::string line;
-  while (!exception_ptr_ && std::getline(in, line))
+  while (!exception_ && std::getline(in, line))
   {
     // since getline does not get the line separator, we have to add it since the parser care
     // about layout in certain cases.
@@ -252,10 +256,10 @@ void SaxParser::parse_stream(std::istream& in)
       firstParseError = parseError;
   }
 
-  if (!exception_ptr_)
+  if (!exception_)
   {
      //This is called just to terminate parsing.
-    const int parseError = xmlParseChunk(context_, 0 /* chunk */, 0 /* size */, 1 /* terminate (1 or 0) */);
+    const int parseError = xmlParseChunk(context_, nullptr /* chunk */, 0 /* size */, 1 /* terminate (1 or 0) */);
 
     if (parseError != XML_ERR_OK && firstParseError == XML_ERR_OK)
       firstParseError = parseError;
@@ -289,10 +293,10 @@ void SaxParser::parse_chunk_raw(const unsigned char* contents, size_type bytes_c
   {
     context_ = xmlCreatePushParserCtxt(
       sax_handler_.get(),
-      0, // user_data
-      0, // chunk
-      0, // size
-      0); // no filename for fetching external entities
+      nullptr,  // user_data
+      nullptr,  // chunk
+      0,        // size
+      nullptr); // no filename for fetching external entities
 
     if(!context_)
     {
@@ -304,7 +308,7 @@ void SaxParser::parse_chunk_raw(const unsigned char* contents, size_type bytes_c
     xmlCtxtResetLastError(context_);
   
   int parseError = XML_ERR_OK;
-  if (!exception_ptr_)
+  if (!exception_)
     parseError = xmlParseChunk(context_, (const char*)contents, bytes_count, 0 /* don't terminate */);
 
   check_for_exception();
@@ -325,10 +329,10 @@ void SaxParser::finish_chunk_parsing()
   {
     context_ = xmlCreatePushParserCtxt(
       sax_handler_.get(),
-      0, // this, // user_data
-      0, // chunk
-      0, // size
-      0); // no filename for fetching external entities
+      nullptr,  // user_data
+      nullptr,  // chunk
+      0,        // size
+      nullptr); // no filename for fetching external entities
 
     if(!context_)
     {
@@ -340,9 +344,9 @@ void SaxParser::finish_chunk_parsing()
     xmlCtxtResetLastError(context_);
 
   int parseError = XML_ERR_OK;
-  if (!exception_ptr_)
+  if (!exception_)
     //This is called just to terminate parsing.
-    parseError = xmlParseChunk(context_, 0 /* chunk */, 0 /* size */, 1 /* terminate (1 or 0) */);
+    parseError = xmlParseChunk(context_, nullptr /* chunk */, 0 /* size */, 1 /* terminate (1 or 0) */);
 
   auto error_str = format_xml_parser_error(context_);
   if (error_str.empty() && parseError != XML_ERR_OK)
@@ -429,7 +433,7 @@ void SaxParserCallback::end_document(void* context)
   auto the_context = static_cast<_xmlParserCtxt*>(context);
   auto parser = static_cast<SaxParser*>(the_context->_private);
 
-  if (parser->exception_ptr_)
+  if (parser->exception_)
     return;
 
   try
@@ -523,15 +527,13 @@ void SaxParserCallback::warning(void* context, const char* fmt, ...)
   auto parser = static_cast<SaxParser*>(the_context->_private);
 
   va_list arg;
-  char buff[1024]; //TODO: Larger/Shared
-
   va_start(arg, fmt);
-  vsnprintf(buff, sizeof(buff)/sizeof(buff[0]), fmt, arg);
+  const Glib::ustring buff = format_printf_message(fmt, arg);
   va_end(arg);
 
   try
   {
-    parser->on_warning(Glib::ustring(buff));
+    parser->on_warning(buff);
   }
   catch (...)
   {
@@ -544,19 +546,17 @@ void SaxParserCallback::error(void* context, const char* fmt, ...)
   auto the_context = static_cast<_xmlParserCtxt*>(context);
   auto parser = static_cast<SaxParser*>(the_context->_private);
 
-  va_list arg;
-  char buff[1024]; //TODO: Larger/Shared
-
-  if (parser->exception_ptr_)
+  if (parser->exception_)
     return;
 
+  va_list arg;
   va_start(arg, fmt);
-  vsnprintf(buff, sizeof(buff)/sizeof(buff[0]), fmt, arg);
+  const Glib::ustring buff = format_printf_message(fmt, arg);
   va_end(arg);
 
   try
   {
-    parser->on_error(Glib::ustring(buff));
+    parser->on_error(buff);
   }
   catch (...)
   {
@@ -570,15 +570,13 @@ void SaxParserCallback::fatal_error(void* context, const char* fmt, ...)
   auto parser = static_cast<SaxParser*>(the_context->_private);
 
   va_list arg;
-  char buff[1024]; //TODO: Larger/Shared
-
   va_start(arg, fmt);
-  vsnprintf(buff, sizeof(buff)/sizeof(buff[0]), fmt, arg);
+  const Glib::ustring buff = format_printf_message(fmt, arg);
   va_end(arg);
 
   try
   {
-    parser->on_fatal_error(Glib::ustring(buff));
+    parser->on_fatal_error(buff);
   }
   catch (...)
   {
